#!/usr/bin/env python3

import traceback
import os

from flask import Flask, render_template, request
from pygments import formatters, highlight, lexers
import markdown

from src import (
    utils,
    config,
)

# TODO: Put the creation of the Flask application into a function
#       https://flask.palletsprojects.com/en/3.0.x/tutorial/factory/
#       def create_app() -> flask.Flask:
app = Flask(
    __name__,
    template_folder="../templates",
    static_folder="../static",
)


def get_tree() -> dict:
    data_path = config.get('data_path')
    ret = {}

    for root, _, files in os.walk(data_path):
        current = ret

        relpath = os.path.relpath(root, data_path)
        breadcrumbs = relpath.split(os.path.sep)
        if breadcrumbs == ['.']:
            breadcrumbs = []

        # Move the current to that folder and create
        # the intermediate steps if they don't exist
        for folder in breadcrumbs:
            if folder not in current or current[folder] is None:
                current[folder] = {}

            current = current[folder]

        print(breadcrumbs)
        for file in files:
            filename, _ = os.path.splitext(file)
            current[filename] = None

    return ret


@app.route('/', defaults={'path': ''})
@app.route('/<path:path>', methods=['POST', 'GET'])
def index(path):
    internal_path = os.path.join(config.get('data_path'), path + '.md')
    path = '/' + path

    # Checks ###################################################

    if '..' in path:
        return 'Path cannot contain double dots, i.e. "..".'

    # If the user sent input, save it ##########################

    if path == '/':
        return (
            render_template(
                "main.html",
                path=path,
                item_list=get_tree(),
                markdown=markdown.markdown,
                content='Welcome',
                edit=False,
            ),
            200,
        )

    raw_markdown = request.form.get('text', None)
    if raw_markdown is not None:
        with open(internal_path, 'w') as file:
            file.write(str(raw_markdown))

    # Get the document contents ################################

    is_edit = 'edit' in request.args and path != '/'

    if os.path.exists(internal_path):
        with open(internal_path) as file:
            raw_markdown = file.read()
    elif is_edit:
        raw_markdown = f'# {path}\n\nChangeme'
    else:
        return (
            render_template(
                "error.html",
                code=404,
                msg=f'The path "{path}" does not exist.',
            ),
            404,
        )

    # Actual serving ###########################################

    return (
        render_template(
            "main.html",
            path=path,
            item_list=get_tree(),
            markdown=markdown.markdown,
            content=raw_markdown,
            edit=is_edit,
        ),
        200,
    )


@ app.errorhandler(Exception)
def internal_error(e):
    tb_text = "".join(traceback.format_exc())

    lexer = lexers.get_lexer_by_name("pytb", stripall=True)
    formatter = formatters.get_formatter_by_name("terminal")
    tb_colored = highlight(tb_text, lexer, formatter)
    print(tb_colored)
    return utils.ERR_500
